@@ -567,3 +567,227 @@ impl ImageImportOptsBuilder {
         url => "url"
     );
 }
+
+impl_opts_builder!(url =>
+    /// Adjust how the image tree is retrieved.
+    ImageTree
+);
+
+impl ImageTreeOptsBuilder {
+    impl_url_bool_field!(
+        /// Show all child images and layers of the specified image.
+        what_requires => "whatrequires"
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how multiple images will be removed.
+    ImagesRemove
+);
+
+impl ImagesRemoveOptsBuilder {
+    impl_url_bool_field!(
+        /// Remove all images.
+        all => "all"
+    );
+
+    impl_url_bool_field!(
+        /// Force image removal (including containers using the images).
+        force => "force"
+    );
+
+    impl_url_vec_field!(
+        /// Images IDs or names to remove.
+        images => "images"
+    );
+}
+
+#[derive(Default, Debug)]
+/// Adjust how an image is pushed to a registry.
+pub struct ImagePushOpts {
+    auth: Option<RegistryAuth>,
+    params: HashMap<&'static str, String>,
+}
+
+impl ImagePushOpts {
+    /// Return a new instance of a builder for ImagePushOpts.
+    pub fn builder() -> ImagePushOptsBuilder {
+        ImagePushOptsBuilder::default()
+    }
+
+    /// Serialize ImagePushOpts as a string. Returns None if no Opts are defined.
+    pub fn serialize(&self) -> Option<String> {
+        if self.params.is_empty() {
+            None
+        } else {
+            Some(crate::util::url::encoded_pairs(
+                self.params.iter().map(|(k, v)| (k, v)),
+            ))
+        }
+    }
+
+    pub(crate) fn auth_header(&self) -> Option<String> {
+        self.auth.clone().map(|a| a.serialize())
+    }
+}
+
+#[derive(Debug, Default)]
+pub struct ImagePushOptsBuilder {
+    auth: Option<RegistryAuth>,
+    params: HashMap<&'static str, String>,
+}
+
+impl ImagePushOptsBuilder {
+    impl_url_str_field!(
+        /// Allows for pushing the image to a different destination than the image refers to.
+        destinations => "destinations"
+    );
+
+    impl_url_bool_field!(
+        /// Require TLS verification.
+        tls_verify => "tlsVerify"
+    );
+
+    pub fn auth(&mut self, auth: RegistryAuth) -> &mut Self {
+        self.auth = Some(auth);
+        self
+    }
+
+    pub fn build(self) -> PullOpts {
+        PullOpts {
+            auth: self.auth,
+            params: self.params,
+        }
+    }
+}
+
+#[derive(Debug)]
+/// Used to filter removed images.
+pub enum ImagePruneFilter {
+    /// When set to true, prune only unused and untagged images. When set to false, all unused
+    /// images are pruned.
+    Dangling(bool),
+    /// Prune images created before this timestamp. The <timestamp> can be Unix timestamps, date
+    /// formatted timestamps, or Go duration strings (e.g. 10m, 1h30m) computed relative to the
+    /// daemon machine’s time.
+    // #TODO: DateTime
+    Until(String),
+    /// Image that contains key label.
+    LabelKey(String),
+    /// Image that contains key-value label.
+    LabelKeyVal(String, String),
+}
+
+impl Filter for ImagePruneFilter {
+    fn query_key_val(&self) -> (&'static str, String) {
+        use ImagePruneFilter::*;
+        match &self {
+            Dangling(dangling) => ("dangling", dangling.to_string()),
+            Until(until) => ("until", until.to_string()),
+            LabelKey(key) => ("label", key.clone()),
+            LabelKeyVal(key, val) => ("label", format!("{}={}", key, val)),
+        }
+    }
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how unused images are removed.
+    ImagePrune
+);
+
+impl ImagePruneOptsBuilder {
+    impl_filter_func!(
+        /// Filters to apply to image pruning.
+        ImagePruneFilter
+    );
+
+    impl_url_bool_field!(
+        /// Remove all images not in use by containers, not just dangling ones.
+        all => "all"
+    );
+
+    impl_url_bool_field!(
+        /// Remove images even when they are used by external containers (e.g, by build
+        /// containers).
+        external => "external"
+    );
+}
+
+#[derive(Debug)]
+/// Used to filter searched images.
+pub enum ImageSearchFilter {
+    IsAutomated(bool),
+    IsOfficial(bool),
+    /// Matches images that has at least 'number' stars.
+    Stars(usize),
+}
+
+impl Filter for ImageSearchFilter {
+    fn query_key_val(&self) -> (&'static str, String) {
+        use ImageSearchFilter::*;
+        match &self {
+            IsAutomated(is_automated) => ("is-automated", is_automated.to_string()),
+            IsOfficial(is_official) => ("is-official", is_official.to_string()),
+            Stars(stars) => ("stars", stars.to_string()),
+        }
+    }
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how to search for images in registries.
+    ImageSearch
+);
+
+impl ImageSearchOptsBuilder {
+    impl_filter_func!(
+        /// Filters to process on the images list.
+        ImageSearchFilter
+    );
+
+    impl_url_field!(
+        /// Maximum number of results.
+        limit: usize => "limit"
+    );
+
+    impl_url_bool_field!(
+        /// List the available tags in the repository.
+        list_tags => "listTags"
+    );
+
+    impl_url_str_field!(
+        /// Term to search for.
+        term => "term"
+    );
+
+    impl_url_bool_field!(
+        /// Skip TLS verification for registries.
+        tls_verify => "tlsVerify"
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how multiple images are exported.
+    ImagesExport
+);
+
+impl ImagesExportOptsBuilder {
+    impl_url_bool_field!(
+        /// Use compression on image.
+        compress => "compress"
+    );
+
+    impl_url_str_field!(
+        /// Format for exported image (only docker-archive is supported).
+        format => "format"
+    );
+
+    impl_url_bool_field!(
+        /// Accept uncompressed layers when copying OCI images.
+        oci_accept_uncompressed_layers => "ociAcceptUncompressedLayers"
+    );
+
+    impl_url_vec_field!(
+        /// References to images to export.
+        references => "references"
+    );
+}
