@@ -2,7 +2,7 @@
 
 use crate::{
     api::{self, ApiResource},
-    conn::{get_http_connector, Headers, Payload, Transport},
+    conn::{get_http_connector, transport, Headers, Payload, Transport},
     models,
     opts::*,
     util, ApiVersion, Error, Result, LATEST_API_VERSION,
@@ -13,12 +13,12 @@ use crate::conn::get_https_connector;
 #[cfg(unix)]
 use crate::conn::get_unix_connector;
 
+use crate::conn::hyper::{body::Bytes, Body, Client, Method, Response};
 use futures_util::{
     io::{AsyncRead, AsyncWrite},
     stream::Stream,
     TryStreamExt,
 };
-use hyper::{body::Bytes, Body, Client, Method, Response};
 use log::trace;
 use serde::de::DeserializeOwned;
 
@@ -101,6 +101,8 @@ impl Podman {
         Self::unix_versioned(socket_path, LATEST_API_VERSION)
     }
 
+    #[cfg(unix)]
+    #[cfg_attr(docsrs, doc(cfg(unix)))]
     /// Same as [`Podman::unix`](Podman::unix) but the API version can be explicitly specified.
     pub fn unix_versioned<P>(socket_path: P, version: impl Into<ApiVersion>) -> Podman
     where
@@ -291,14 +293,36 @@ impl Podman {
     api_doc! {
     System => InfoLibpod
     /// Returns information on the system and libpod configuration
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.info().await {
+    ///     Ok(info) => println!("{:?}", info),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
     |
-    pub async fn info(&self) -> Result<models::HostInfo> {
+    pub async fn info(&self) -> Result<models::Info> {
         self.get_json("/libpod/info").await
     }}
 
     api_doc! {
     System => Ping
     /// Return protocol information from libpod.
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.ping().await {
+    ///     Ok(info) => println!("{:?}", info),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
     |
     pub async fn ping(&self) -> Result<models::LibpodPingInfo> {
         self.get("/libpod/_ping")
@@ -309,6 +333,17 @@ impl Podman {
     api_doc! {
     System => VersionLibpod
     /// Returns component version information.
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.version().await {
+    ///     Ok(info) => println!("{:?}", info),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
     |
     pub async fn version(&self) -> Result<models::LibpodVersionResponse> {
         self.get_json("/libpod/version").await
@@ -317,6 +352,17 @@ impl Podman {
     api_doc! {
     System => DataUsageLibpod
     /// Return information about disk usage for containers, images, and volumes.
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.data_usage().await {
+    ///     Ok(info) => println!("{:?}", info),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
     |
     pub async fn data_usage(&self) -> Result<models::LibpodDataUsageResponse> {
         self.get_json("/libpod/system/df").await
@@ -325,6 +371,17 @@ impl Podman {
     api_doc! {
     System => PruneLibpod
     /// Prune unused data.
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.prune().await {
+    ///     Ok(info) => println!("{:?}", info),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
     |
     pub async fn prune(&self) -> Result<models::LibpodSystemPruneResponse> {
         self.post_json("/libpod/system/prune", Payload::empty())
@@ -335,6 +392,22 @@ impl Podman {
     System => EventsLibpod
     |
     /// Returns system events
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// use futures_util::StreamExt;
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// let events = podman.events(&Default::default());
+    ///
+    /// while let Some(event) = events.next().await {
+    ///     match event {
+    ///         Ok(event) => println!("{:?}", event),
+    ///         Err(e) => eprintln!("{}", e),
+    ///     }
+    /// }
+    /// ```
     pub fn events<'libpod>(
         &'libpod self,
         opts: &EventsOpts,
@@ -355,6 +428,69 @@ impl Podman {
         )
     }}
 
+    api_doc! {
+    Play => KubeLibpod
+    /// Create and run pods based on a Kubernetes YAML file (pod or service kind).
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// let yaml = r#"
+    /// apiVersion: v1
+    /// kind: Pod
+    /// metadata:
+    ///   name: youthfulwescoff
+    /// spec:
+    ///   containers:
+    ///   - image: docker.io/library/alpine:latest
+    ///     name: youthfulwescoff
+    ///     securityContext:
+    ///       capabilities:
+    ///         drop:
+    ///         - CAP_MKNOD
+    ///         - CAP_NET_RAW
+    ///         - CAP_AUDIT_WRITE
+    ///     stdin: true
+    ///     tty: true
+    /// "#;
+    ///
+    /// match podman.play_kubernetes_yaml(&Default::default(), yaml).await {
+    ///     Ok(report) => println!("{:?}", report),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
+    |
+    pub async fn play_kubernetes_yaml(
+        &self,
+        opts: &PlayKubernetesYamlOpts,
+        yaml: impl Into<String>,
+    ) -> Result<models::PlayKubeReport> {
+        let ep = util::url::construct_ep("/libpod/play/kube", opts.serialize());
+        let yaml = yaml.into();
+        self.post_json(&ep, Payload::Text(yaml)).await
+    }}
+
+    api_doc! {
+    Play => KubeDownLibpod
+    /// Tears down pods defined in a YAML file.
+    ///
+    /// Example:
+    ///
+    /// ```no_run
+    /// let podman = Podman::unix("/run/user/1000/podman/podman.sock");
+    ///
+    /// match podman.remove_kubernetes_pods().await {
+    ///     Ok(report) => println!("{:?}", report),
+    ///     Err(e) => eprintln!("{}", e),
+    /// }
+    /// ```
+    |
+    pub async fn remove_kubernetes_pods(&self) -> Result<models::PlayKubeReport> {
+        self.delete_json("/libpod/play/kube").await
+    }}
+
     pub(crate) async fn resource_exists(
         &self,
         resource: ApiResource,
@@ -365,7 +501,7 @@ impl Podman {
             Ok(_) => Ok(true),
             Err(e) => match e {
                 crate::Error::Fault {
-                    code: http::StatusCode::NOT_FOUND,
+                    code: crate::conn::http::StatusCode::NOT_FOUND,
                     message: _,
                 } => Ok(false),
                 e => Err(e),
@@ -373,6 +509,29 @@ impl Podman {
         }
     }
 
+    pub(crate) async fn generate_systemd_units(
+        &self,
+        opts: &SystemdUnitsOpts,
+        id: &crate::Id,
+    ) -> Result<serde_json::Value> {
+        let ep = util::url::construct_ep(
+            format!("/libpod/generate/{}/systemd", &id),
+            opts.serialize(),
+        );
+        self.get_json(&ep).await
+    }
+
+    pub(crate) async fn generate_kube_yaml(&self, service: bool, id: &crate::Id) -> Result<String> {
+        let opts = [("names", id.to_string()), ("service", service.to_string())];
+        let ep = util::url::construct_ep(
+            "/libpod/generate/kube",
+            Some(util::url::encoded_pairs(opts)),
+        );
+
+        let body = self.get(&ep).await.map(|b| b.into_body())?;
+        transport::body_to_string(body).await.map_err(Error::from)
+    }
+
     //####################################################################################################
     //
     // Utility functions to make requests
@@ -388,6 +547,7 @@ impl Podman {
                 Headers::none(),
             )
             .await
+            .map_err(Error::from)
     }
 
     pub(crate) async fn get_json<T: DeserializeOwned>(&self, endpoint: &str) -> Result<T> {
@@ -417,6 +577,7 @@ impl Podman {
                 Headers::none(),
             )
             .await
+            .map_err(Error::from)
     }
 
     #[allow(dead_code)]
@@ -424,7 +585,7 @@ impl Podman {
         &self,
         endpoint: &str,
         body: Payload<B>,
-        headers: Headers,
+        headers: Option<Headers>,
     ) -> Result<String>
     where
         B: Into<Body>,
@@ -434,9 +595,10 @@ impl Podman {
                 Method::POST,
                 self.version.make_endpoint(endpoint),
                 body,
-                Some(headers),
+                headers,
             )
             .await
+            .map_err(Error::from)
     }
 
     #[allow(dead_code)]
@@ -452,6 +614,7 @@ impl Podman {
                 Headers::none(),
             )
             .await
+            .map_err(Error::from)
     }
 
     pub(crate) async fn post_json<B, T>(
@@ -511,6 +674,7 @@ impl Podman {
                 Headers::none(),
             )
             .await
+            .map_err(Error::from)
     }
 
     pub(crate) async fn delete_json<T: DeserializeOwned>(&self, endpoint: &str) -> Result<T> {
@@ -538,6 +702,7 @@ impl Podman {
                 Headers::none(),
             )
             .await
+            .map_err(Error::from)
     }
 
     /// Send a streaming post request.
@@ -552,12 +717,14 @@ impl Podman {
     where
         B: Into<Body> + 'a,
     {
-        self.transport.stream_chunks(
-            Method::POST,
-            self.version.make_endpoint(endpoint),
-            body,
-            headers,
-        )
+        self.transport
+            .stream_chunks(
+                Method::POST,
+                self.version.make_endpoint(endpoint),
+                body,
+                headers,
+            )
+            .map_err(Error::from)
     }
 
     #[allow(dead_code)]
@@ -571,12 +738,14 @@ impl Podman {
     where
         B: Into<Body> + 'a,
     {
-        self.transport.stream_json_chunks(
-            Method::POST,
-            self.version.make_endpoint(endpoint),
-            body,
-            headers,
-        )
+        self.transport
+            .stream_json_chunks(
+                Method::POST,
+                self.version.make_endpoint(endpoint),
+                body,
+                headers,
+            )
+            .map_err(Error::from)
     }
 
     #[allow(dead_code)]
@@ -612,12 +781,14 @@ impl Podman {
         &'a self,
         endpoint: impl AsRef<str> + Unpin + 'a,
     ) -> impl Stream<Item = Result<Bytes>> + 'a {
-        self.transport.stream_chunks(
-            Method::GET,
-            self.version.make_endpoint(endpoint),
-            Payload::empty(),
-            Headers::none(),
-        )
+        self.transport
+            .stream_chunks(
+                Method::GET,
+                self.version.make_endpoint(endpoint),
+                Payload::empty(),
+                Headers::none(),
+            )
+            .map_err(Error::from)
     }
 
     pub(crate) fn stream_get_json<'a, T>(
@@ -653,6 +824,7 @@ impl Podman {
         self.transport
             .stream_upgrade(Method::POST, self.version.make_endpoint(endpoint), body)
             .await
+            .map_err(Error::from)
     }
 }
 
@@ -667,9 +839,10 @@ mod tests {
         d.unwrap();
 
         #[cfg(unix)]
-        let d = Podman::new("unix://127.0.0.1:80");
-        d.unwrap();
-
+        {
+            let d = Podman::new("unix://127.0.0.1:80");
+            d.unwrap();
+        }
         #[cfg(not(unix))]
         {
             let d = Podman::new("unix://127.0.0.1:80");
